from group_center.core.feature.machine_user_message import machine_user_message_directly

machine_user_message_directly("konghaomin", "[Machine User Message]Test passed!")
machine_user_message_directly("konghaomin", "[Machine User Message]Test passed again!")
